#include "labyrinthe.h"
#include <Imagine/Graphics.h>
using namespace Imagine;

// Constructeur: on menage l'entree et la sortie, sinon les murs sont pleins
Labyrinthe::Labyrinthe(int largeur, int hauteur) {
    w = largeur;
    h = hauteur;
    int n = (w+1)*(h+1);
    mur = new Mur[n];
    // Ouvertures
    mur[0].hor = false; //entree
    mur[n-2].hor = false; //sortie
    // Les murs superflus
    for(int y=0; y<=hauteur; y++)
        mur[y*(w+1)+w].hor = false;
    for(int x=0; x<=largeur; x++)
        mur[h*(w+1)+x].ver = false;
}

// Destructeur
Labyrinthe::~Labyrinthe() {
    delete [] mur;
}

// Affichage
void Labyrinthe::affiche(int dx, int dy) const {
    for(int y=0; y<=h; y++)
        for(int x=0; x<=w; x++) {
            if(mur[y*(w+1)+x].hor)
                drawLine(dx+x*zoom, dy+y*zoom, dx+(x+1)*zoom, dy+y*zoom, RED);
            if(mur[y*(w+1)+x].ver)
                drawLine(dx+x*zoom, dy+y*zoom, dx+x*zoom, dy+(y+1)*zoom, RED);
        }
}

// Enleve tous les murs interieurs
void Labyrinthe::vide() {
    for(int y=0; y<h; y++)
        for(int x=0; x<w; x++) {
            if(y!=0)
                mur[y*(w+1)+x].hor = false;
            if(x!=0)
                mur[y*(w+1)+x].ver = false;
        }
}

// Generation d'un labyrinthe
void Labyrinthe::genere() {
    vide();
    genere(0, 0, w-1, h-1);
}

// Generation recursive du labyrinthe
void Labyrinthe::genere(int x0, int y0, int x1, int y1) {
    int nh=x1-x0, nv=y1-y0;
    if(nh<=0 || nv<=0)
        return;
    int cut = rand() % (nh+nv);
    if(cut<nh) { // Coupure verticale
        int x = x0+cut+1;
        for(int y=y0; y<=y1; y++)
            mur[y*(w+1)+x].ver = true;
        int y=y0+rand()%(nv+1); // Ouverture
        mur[y*(w+1)+x].ver = false;
        genere(x0,y0,x-1,y1);
        genere(x,y0,x1,y1);
    } else { // Coupure horizontale
        int y = y0+(cut-nh)+1;
        for(int x=x0; x<=x1; x++)
            mur[y*(w+1)+x].hor = true;
        int x=x0+rand()%(nh+1);  // Ouverture
        mur[y*(w+1)+x].hor = false;
        genere(x0,y0,x1,y-1);
        genere(x0,y,x1,y1);
    }
}

// Bouche l'impasse en (x,y) (si c'en est une), et va au voisin qui en sort.
bool Labyrinthe::bouche_impasse(int& x, int& y) {
    int dx=1, dy=0;
    bool dirs[4]; // voisins inaccessibles
    dirs[0] = mur[y*(w+1)+x].hor;     // haut
    dirs[1] = mur[y*(w+1)+x].ver;     // gauche
    dirs[2] = mur[(y+1)*(w+1)+x].hor; // bas
    dirs[3] = mur[y*(w+1)+x+1].ver;   // droite
    bool impasse=false;
    for(int i=0; i<4; i++)
        if(!dirs[i]) {
            if(impasse) // on verifie qu'il n'y a pas plus d'une sortie
                return false;
            impasse = true;
        }
    if(!dirs[0]) mur[y--*(w+1)+x].hor = true;
    if(!dirs[1]) mur[y*(w+1)+x--].ver = true;
    if(!dirs[2]) mur[++y*(w+1)+x].hor = true;
    if(!dirs[3]) mur[y*(w+1)+ ++x].ver = true;
    return impasse;
}

// Peut-on aller de (x,y) dans la direction (horizontale ou verticale) (dx,dy)?
bool Labyrinthe::adjacent(int x, int y, int dx, int dy) const {
    if(dx != 0) {
        assert(dy==0);
        if(dx>0) ++x;
        return !mur[y*(w+1)+x].ver;
    }
    assert(dy!=0);
    if(dy>0) ++y;
    return !mur[y*(w+1)+x].hor;
}
