// Imagine++ project
// Project:  Maze generation & solving
// Author:   Pascal Monasse
// Date:     2019/01/11

#include <Imagine/Graphics.h>
#include "labyrinthe.h"
using namespace Imagine;

const int marge=10; // Pour affichage

struct point { int x,y; };

// Resolution par bouchage successif des impasses
void resolution_impasses(Labyrinthe& L) {
    for(int y=0; y<L.hauteur(); y++)
        for(int x=0; x<L.largeur(); x++) {
            point pt = {x,y}, pt0=pt;
            while(L.bouche_impasse(pt.x, pt.y)) {
                fillCircle(marge+pt0.x*zoom+zoom/2, marge+pt0.y*zoom+zoom/2,
                         zoom/2-2, BLUE);
                pt0 = pt;
            }
        }
}

// Exploration recursive
bool explore(point o, point pt, const Labyrinthe& L, bool* dejavu) {
    dejavu[pt.y*L.largeur()+pt.x] = true;
    fillCircle(marge+o.x*zoom+zoom/2, marge+o.y*zoom+zoom/2, zoom/2-2, BLACK);
    fillCircle(marge+pt.x*zoom+zoom/2, marge+pt.y*zoom+zoom/2, zoom/2-2, GREEN);
    milliSleep(10);
    int dx=1, dy=0;
    for(int i=0; i<4; i++) {
        point q = {pt.x+dx, pt.y+dy};
        if(q.x>=0 && q.y>=0 && L.adjacent(pt.x, pt.y, dx, dy)) {
            if(q.x>=L.largeur() || q.y>=L.hauteur())
                return true;
            if(!dejavu[q.y*L.largeur()+q.x] && explore(pt, q, L, dejavu))
                return true;
        }
        // Rotation du vecteur de deplacement
        int tmp=dx; dx = dy; dy = -tmp;
    }
    fillCircle(marge+pt.x*zoom+zoom/2, marge+pt.y*zoom+zoom/2, zoom/2-2, BLUE);
    fillCircle(marge+o.x*zoom+zoom/2, marge+o.y*zoom+zoom/2, zoom/2-2, GREEN);
    milliSleep(10);
    return false;
}

// Resolution par exploration
void resolution_exploration(const Labyrinthe& L) {
    bool* dejavu = new bool[L.largeur()*L.hauteur()];
    for(int i=0; i<L.largeur()*L.hauteur(); i++)
        dejavu[i] = 0;
    // On part de la case (0,0)
    point pt = {0,0}, origine=pt;
    explore(origine, pt, L, dejavu);
    delete [] dejavu;
}

int main() {
    srand((unsigned int)time(NULL));
    const int largeur=30, hauteur=20;
    openWindow(2*marge+largeur*zoom+1, 2*marge+hauteur*zoom+1, "Amazing Maze");
    Labyrinthe L(largeur, hauteur);

    L.affiche(marge, marge);
    click(); clearWindow();

    L.vide();
    L.affiche(marge, marge);
    click(); clearWindow();

    L.genere();
    L.affiche(marge, marge);
    click();

    resolution_impasses(L);
    click(); clearWindow();

    L.genere();
    L.affiche(marge, marge);
    resolution_exploration(L);

    endGraphics();
    return 0;
}
